%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Satelite
% posizione sat
% velocit sat
% quantit di moto complessiva


D=1;
Js=1;
Jv=1;
Km=2;


Asat=[0 1 0;
     0 -D*(Js+Jv)/(Js*Jv) D/(Js*Jv);
     0 0 0];
 
Bsat=[0 -Km/Js 0]';

Csat=[1 0 0];


% Processo
A=[0 1;
     0 -D*(Js+Jv)/(Js*Jv)];
 
B=[0 -Km/Js]';

C=[-1 0];

% Esosistema
S=[0];


P=[0;1];
Q=[1];

% Dimensioni 
n=max(size(A));
ns=max(size(S));

% Test sul Rango 
Arank=[A B; C 0];
if (rank(Arank)<min(size(Arank)))
    disp('Non risolvibile per caduta di rango')
    return
end

% Test sulla controllabilit di (A,B)
R=[B A*B];
if det(R)==0
    disp('Non risolvibile per mancanza di controllabilit')
    return
end


%calcolo dei guadagni per il sistema e per l'osservatore
K=acker(A,-B,[-2 -2])

disp('Autovalori del sistema:')
eig(A+B*K)'


Ao=[A P; zeros(ns,n) S];
Co=[C Q];
Obs=[Co; Co*Ao ; Co*Ao*Ao];
if det(Obs)==0
    disp('Manca l''osservabilit')
    return
end



G=acker([A P; zeros(ns,n) S]',[C Q]',[-3 -3 -3])'


disp('Autovalori dell''osservatore:')
eig([A P; zeros(ns,n) S]-G*[C Q])



%Soluzione semplificata delle equazioni del regolatore
Xf=inv([A B; C 0])*[-P;-Q]
Pigreco=Xf(1:2,:);
Gamma=Xf(3,:);



% Calcolo del compensatore
H=[K Gamma-K*Pigreco];
F=[A P;  zeros(ns,n) S]+[B;zeros(ns,1)]*H-G*[C Q];

%--  Verifca -----
% Autovalori del ciclo chiuso
disp('Autovalori del ciclo chiuso:')
eig([A B*H; G*C F])

%Verifica
Sigma=[Pigreco;eye(ns)]

if sum(sum(abs(Sigma*S-F*Sigma)))>1e-10
    disp('Equazioni di Francis non corrette!')
    Sigma*S-F*Sigma
    return
end


% Simulazione del sistema complessivo
Ac=[ [A B*H; G*C F] [P; G*Q]; zeros(ns,2*n+ns)  S];
Bc=zeros(2*n+2*ns,1);
Cc=[C zeros(1,n+ns) Q];

Cy=[-C zeros(1,n+ns) [0]];
Crif=[zeros(1,n) zeros(1,n+ns) [1]];


Dc=0;

SYS = ss(Ac,Bc,Cc,Dc);
max_t_sim=6;
t_sim=[0:0.01:max_t_sim]';
u_sim=zeros(max_t_sim/0.01+1,1);


[Y,T,X] = lsim(SYS,u_sim,t_sim,[0 0 0 0 0 1]);

figure(1)

subplot(211)
plot(T,[(Cy*X')'  (Crif*X')'])
ylabel('Uscita e riferimento')
grid


subplot(212)
plot(T,X(:,1:6))
grid

figure(2)
plot(T,[(Cy*X')'  (Crif*X')'])
title('Uscita e Riferimento')
xlabel('sec')
ylabel('rad')
grid


%-------------------------------------------------------------------
% Regolatore discreto

Tc=0.1;
RegDiscr=c2d(ss(F,G,H,0),Tc,'Tustin');
